package Vulture::TransHandler;

use strict;
use warnings;

use Apache2::Const -compile => qw(OK HTTP_UNAUTHORIZED REDIRECT);
use Apache2::RequestRec ();
use Apache2::RequestIO ();
use APR::Table;  
use HTTP::Request;
use Apache2::RequestUtil();
use LWP::UserAgent;

use Vulture qw(&get_app $proto &read_conf $db_uri &get_cookie &session);

sub handler {
	my $r = shift;
	
	my $uri = $r->uri;
	my $server;
	my $dbh = DBI->connect($db_uri); 
	my $app = get_app($r->hostname, $dbh, $r->dir_config('if_id'));
	if ($app and $app->{up} and $uri ne "/favicon.ico") {
		my ($id) = get_cookie($r->headers_in->{Cookie}, 'vulture=([^;]*)');
		my %session;
		session(\%session, $id) if ($id);

		if ($session{post} or !$app->{'auth'} or $app->{'auth_type'} == 2) {
			if ($session{posted}
			    or !$app->{'auth'}
			    or $app->{'auth_type'} == 2
			    or ($session{post}
				and $app->{'sso_forward_type'} eq "basic")) {

				$server = $app->{'url'};
				
				my $proxy_rules 	= $app->{'rewrite'};
				my $do_redirect 	= 0;
				my $do_proxy 		= 0;

				if ($proxy_rules){

					$proxy_rules =~ s/\r//;

					foreach (split ("\n",$proxy_rules)) {
						$do_redirect = 0;

						my ($exp,$rule) = split (/\s*=>\s*/);
						next if ($exp eq "" or $rule eq "");
						next unless ($uri =~ /$exp/);
						
						$rule =~ s/\s*//;						
						$exp =~ s/\s*//;						

						if ($rule =~ /\[P\]/ ) {
							$rule =~ s/\s*\[P\]//;
						}
						elsif ($rule =~ /\[R\]/ ) {
							$rule =~ s/\s*\[R\]//;
							$do_redirect = 1;
						}
						elsif ($uri =~ /$exp/x and $rule =~ /\[([0-9]*)\]/ ) {
							$r->status ($1);
							return $r->status;
						}

						if ($rule=~/\$[0-9]/ and $uri =~ /$exp/x ) {
							my @tab = split ('\$[0-9]', $rule);
							my $i=1;
							my $tmp;
							foreach (@tab) {
								$tmp.="'".$_."'";
								last if ($i == $#tab+1) ;
								$tmp.='.$'.$i.'.';
								$i++;
							}
							$tmp = eval ($tmp);
							$uri =~ s|$exp|$tmp|x;
							$do_redirect++;
							last;
						}
						elsif ($uri =~ /$exp/x ) {
							$uri =~ s|$exp|$rule|x;
							$do_redirect++;
							last;
						}
					}
				}

				if ($do_redirect == 2) {
					$r->status(200);
					if ($uri =~ "://") {
						$r->headers_out->set('Location' => $uri);
					}
					else {
						$r->headers_out->set('Location' => $proto."://". $r->hostname .":".$r->get_server_port ."$uri");
					}
					return Apache2::Const::REDIRECT;
				}

				if ($app->{'sso_forward_type'} eq "basic") {
				    $r->headers_in->set('Authorization' => $session{post});
				}
				
				foreach (keys %{$app->{'headers'}}) {
				    $r->headers_in->set($_ => $app->{'headers'}->{$_});
				}
				$r->filename("proxy:$server$uri");
				$r->handler('proxy-server');
				$r->proxyreq(1);
				return Apache2::Const::OK;
			}
			my $request = HTTP::Request->new('POST', $app->{post_url}, undef, $session{post});
			$session{posted} = 1;
			$request->push_header('Content-Type' => 'application/x-www-form-urlencoded');
			my $ua = LWP::UserAgent->new;
			my $response = $ua->request($request); 
			if ($response->headers->header('Set-Cookie')) {
				foreach ($response->headers->header('Set-Cookie')) {
					$r->headers_out->add('Set-Cookie' => $_ . "; path=/");
				}
			}
		}
	}
	$r->status(Apache2::Const::REDIRECT);
	return Apache2::Const::OK;
}


1;
