<?php
/*
* $Header: /projects/vultureng/www/phpmvc/WEB-INF/classes/phpmvc/dbcp/PearPgsqlDataSource.php,v 1.1.1.1 2005/03/07 08:20:45 vultureng Exp $
* $Revision: 1.1.1.1 $
* $Date: 2005/03/07 08:20:45 $
*
* ====================================================================
*
* License:	GNU Lesser General Public License (LGPL)
*
* Copyright (c) 2002, 2003 John C.Wildenauer.  All rights reserved.
*
* This file is part of the php.MVC Web applications framework
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.

* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.

* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


include_once 'lib/pear/DB/pgsql.php';
include_once 'lib/pear/DB.php';

/**
* A class that provides an interface to the Pear::DB database abstraction classes.
*
* @author John C. Wildenauer
*
* @version $Revision: 1.1.1.1 $
* @public
*/
class PearPgsqlDataSource extends DB_pgsql {

	// ----- Properties ----------------------------------------------------- //

	var $phptype		= ''; 	// Database backend (mysql, ...). Not reqd here.
	var $dbsyntax		= ''; 	// Database used with regards to SQL syntax etc.
	var $protocol		= ''; 	// Communication protocol to use (tcp, unix etc.)
	var $hostspec		= ''; 	// Host specification (hostname[:port])
	var $database		= ''; 	// Database to use on the DBMS server
	var $username		= ''; 	// User name for login
	var $password		= ''; 	// Password for login
	var $persistent	= False;	// Boolean


	// Setters. ( set[PropertyToSet]($value) )
	// <set-property property = "username"   value = "xxxxxxxx"/>

	/**
	* Database used with regards to SQL syntax etc.
	* <p><pre>&lt;set-property property = "dbsyntax"   value = "xxxxxxxx"/&gt;</pre>
	*
	* @param string	Database used with regards to SQL syntax etc.
	* @returns void
	*/
	function setDbsyntax($value) {
		$this->dbsyntax = $value;
	}

	/**
	* Communication protocol to use (tcp, unix etc.)
	*
	* @param string	The communication protocol to use (tcp, unix etc.)
	* @returns void
	*/
	function setProtocol($value) {
		$this->protocol = $value;
	}

	/**
	* Host specification (hostname[:port])
	*
	* @param string	The hostname[:port] for connection to the DBMS.
	* @returns void
	*/
	function setHost($value) {
		$this->hostspec = $value;
	}

	/**
	* Database to use on the DBMS server
	*
	* @param string	The database to use on the DBMS server.
	* @returns void
	*/
	function setDatabase($value) {
		$this->database = $value;
	}

	/**
	* User name for login
	*
	* @param string	The username for this connection.
	* @returns void
	*/
	function setUsername($value) {
		$this->username = $value;
	}

	/**
	* Password for login
	*
	* @param string	The password for this connection.
	* @returns void
	*/
	function setPassword($value) {
		$this->password = $value;
	}

	/**
	* Persistent, expects a boolean (Default is False)
	*
	* @param string	Whether to use a persistent connection.
	* @returns void
	*/
	function setPersistent($value) {
		if(strtolower($value) == 'true') {
			$this->persistent = True;
		}
	}


// ----- Public Methods ---------------------------------------------------- //

	/**
	* Setup the database server connection.
	*<p>Returns immediately if a connection already exists.
	*<p>Note: It is assumed that this object has been pre-configured via the
	* initialisation process.
	*
	* @returns void
	*/
	function open() {

		if($this->connection != '') {
			// We already have a valid connection
			return;
		}

		$dsninfo = array();
		$dsninfo['dbsyntax'] = $this->dbsyntax;
		$dsninfo['protocol'] = $this->protocol;	
		$dsninfo['hostspec'] = $this->hostspec;
		$dsninfo['database'] = $this->database;
		$dsninfo['username'] = $this->username;
		$dsninfo['password'] = $this->password;
		$persistent				= $this->persistent;

		$conn = $this->connect($dsninfo, $persistent);

		// Do we have a vaild commection
		if( DB::isError($conn) ) {
			die( $conn->getMessage() );
		}

	}

}
?>